/*:
 * @target MZ
 * @plugindesc 歩行画像合成プラグイン
 * @author GrayOgre
 * @url https://grayogre.info/rmmz/plugin/index.html
 * @help
 * 
 * このプラグインは歩行グラフィックの合成を行います。
 * 
 * このプラグインは以下のプラグインコマンドで構成されます。
 * (1) 合成
 *   複数の歩行グラフィックを合成します。
 *   合成は最初に指定されたグラフィックに上書きする形で合成されます。
 *   ファイル名には変数置換(\V[変数番号])が使えます。
 *   合成データのファイル名はリストの最初のファイル名になります。
 *   (ツクール本体では存在しない画像を指定できないため)
 *   合成データの１番目のデータを使い回す場合は、
 *   空のデータを用意してそれに合成するようにしてください。
 * 
 * このプラグインを使用するためには、
 * あらかじめ公式プラグイン PluginCommonBase.js を組み込んでおく必要があります。
 * 
 * Copyright (c) 2021 GrayOgre
 * Released under the MIT license
 * https://opensource.org/licenses/mit-license.php
 * 
 * Version 1.0.0
 * 
 * @base PluginCommonBase
 * @orderAfter PluginCommonBase
 * 
 * @command merge
 *   @text 合成
 *   @desc グラフィックの合成を行う
 *   @arg list
 *     @text グラフィック一覧
 *     @desc 合成するグラフィックの一覧
 *     @type struct<graphInfo>[]
 */
/*~struct~graphInfo:
 * @param filename
 *   @text 画像ファイル名
 *   @desc 合成するグラフィックのファイル名
 *   @type file
 *   @dir img/characters
 * @param index
 *   @text グラフィックインデックス
 *   @desc 合成するグラフィックの画像内の順序位置
 *   @type number
*/

(() => {
    'use strict';

    let bitmapCache = null;

    const _createDataObjects_func = DataManager.createGameObjects;
    DataManager.createGameObjects = (() => {
        _createDataObjects_func.call(this);
        bitmapCache = {};
    });

    const _makeSaveContents_func = DataManager.makeSaveContents;
    DataManager.makeSaveContents = (() => {
        const contents = _makeSaveContents_func.call(this);
        contents.gog_bitmapCache = bitmapCache;
        return contents;
    });

    const _extractSaveContents_func = DataManager.extractSaveContents;
    DataManager.extractSaveContents = ((contents) => {
        _extractSaveContents_func.call(this, contents);
        bitmapCache = contents.gog_bitmapCache;
    });

    // 合成
    PluginManagerEx.registerCommand(document.currentScript, "merge", args => {
        if (args.list.length <= 0) {
            return;
        }
        const list = args.list;
        const filenames = [];
        for (let i = 0; i < list.length; i++) {
            list[i].filename = list[i].filename.replace(/\\V\[(\d+)\]/g,(a,b) => $gameVariables.value(Number(b)));
            list[i].index 
        }
        const baseFile = list[0].filename;
        const bitmaps = [];
        let firstSkip = false;
        if (bitmapCache[baseFile]) {
            bitmaps.push(bitmapCache[baseFile]);
            firstSkip = true;
        }
        bitmapCache[baseFile] = null;
        for (let i = 0; i < list.length; i++) {
            if (firstSkip && i === 0) {
                continue;
            }
            const bitmap = ImageManager.loadCharacter(list[i].filename);
            bitmaps.push(bitmap);
        }
        for (let i = 0; i < bitmaps.length; i++) {
            bitmaps[i].addLoadListener(() => {
                if(!(bitmapCache[baseFile]) && bitmaps.every(b => b.isReady())) {
                    mergeBitmap(bitmaps, list);
                }
            });
        }
    });

    // ビットマップ合成
    function mergeBitmap(bitmaps, list) {
        const baseBitmap = bitmaps[0];
        const baseName = list[0].filename;
        const baseIndex = Number(list[0].index);
        let width = baseBitmap.width;
        let height = baseBitmap.height;
        let base_x = 0;
        let base_y = 0;
        if (!ImageManager.isBigCharacter(baseName)) {
            width = baseBitmap.width / 4;
            height = baseBitmap.height / 2;
            base_x = (baseIndex % 4) * width;
            base_y = Math.floor(baseIndex / 4) * height;
        }
        for (let i = 1; i < list.length; i++) {
            let x = 0;
            let y = 0;
            if (!ImageManager.isBigCharacter(list[i].filename)) {
                const index = Number(list[i].index);
                x = (index % 4) * width;
                y = Math.floor(index / 4) * height;
            }
            baseBitmap.blt(bitmaps[i], x, y, width, height, base_x, base_y);
            baseBitmap._baseTexture.update();
        }
        bitmapCache[baseName] = baseBitmap;
    }

    // ここで合成済みのビットマップにすり替える
    const _sprite_Character_setCharacterBitmap = Sprite_Character.prototype.setCharacterBitmap;
    Sprite_Character.prototype.setCharacterBitmap = function() {
        const charaName = this._characterName;
        if (bitmapCache[charaName]) {
            this.bitmap = bitmapCache[charaName];
            this._isBigCharacter = ImageManager.isBigCharacter(charaName);
        } else {
            _sprite_Character_setCharacterBitmap.call(this);
        }
    }

})();